"""
Hook into Alliance Auth
"""

# Alliance Auth
from allianceauth import hooks

# Memberaudit Securegroups
from memberaudit_securegroups.models import (
    ActivityFilter,
    AgeFilter,
    AssetFilter,
    ComplianceFilter,
    SkillPointFilter,
    SkillSetFilter,
)


@hooks.register("secure_group_filters")
def filters():
    return [
        ActivityFilter,
        AgeFilter,
        AssetFilter,
        ComplianceFilter,
        SkillPointFilter,
        SkillSetFilter,
    ]



"""
The models
"""

# Standard Library
import datetime

# Third Party
import humanize

# Django
from django.contrib.auth.models import User
from django.core.cache import cache
from django.db import models
from django.db.models import Q
from django.utils.translation import gettext_lazy as _

# Member Audit
from memberaudit.models import Character, CharacterAsset, General, SkillSet

# Alliance Auth (External Libs)
from eveuniverse.models import EveType


class SingletonModel(models.Model):
    """
    SingletonModel
    """

    class Meta:
        """
        Model meta definitions
        """

        abstract = True

    def delete(self, *args, **kwargs):
        """
        delete action
        :param args:
        :param kwargs:
        :return:
        """

        pass

    def set_cache(self):
        """
        Setting cache
        :return:
        """

        cache.set(self.__class__.__name__, self)

    def save(self, *args, **kwargs):
        """
        save action
        :param args:
        :param kwargs:
        :return:
        """

        self.pk = 1
        super().save(*args, **kwargs)

        self.set_cache()

    @classmethod
    def load(cls):
        """
        Get cache
        :return:
        """

        if cache.get(cls.__name__) is None:
            obj, created = cls.objects.get_or_create(pk=1)

            if not created:
                obj.set_cache()

        return cache.get(cls.__name__)


class BaseFilter(models.Model):
    """
    BaseFilter
    """

    description = models.CharField(
        max_length=500,
        help_text=_("The filter description that is shown to end users."),
    )  # this is what is shown to the user

    class Meta:
        """
        Model meta definitions
        """

        abstract = True

    def __str__(self):
        """
        Model stringified name
        :return:
        """

        return f"{self.name}: {self.description}"

    @property
    def name(self):
        """
        Filter name
        :return:
        """

        return "Compliance"

    def process_filter(self, user: User):
        """
        This is the check run against a users characters
        :param user:
        :return:
        """

        raise NotImplementedError("Please Create a filter!")


class ActivityFilter(BaseFilter):
    """
    ActivityFilter
    """

    inactivity_threshold = models.PositiveIntegerField(
        help_text=_("Maximum allowable inactivity, in <strong>days</strong>."),
    )

    @property
    def name(self):
        """
        Filter name
        :return:
        """

        return f"Activity [days={self.inactivity_threshold}]"

    def process_filter(self, user: User):
        """
        Processing filter
        :param user:
        :return:
        """

        threshold_date = datetime.datetime.now(
            datetime.timezone.utc
        ) - datetime.timedelta(days=self.inactivity_threshold)

        return (
            Character.objects.owned_by_user(user=user)
            .filter(
                Q(online_status__last_login__gt=threshold_date)
                | Q(online_status__last_logout__gt=threshold_date),
            )
            .count()
            > 0
        )


class AgeFilter(BaseFilter):
    """
    AgeFilter
    """

    age_threshold = models.PositiveIntegerField(
        help_text=_("Minimum allowable age, in <strong>days</strong>."),
    )

    @property
    def name(self):
        """
        Filter name
        :return:
        """

        return f"Member Audit Age [days={self.age_threshold}]"

    def process_filter(self, user: User):
        """
        Processing filter
        :param user:
        :return:
        """

        threshold_date = datetime.datetime.now(
            datetime.timezone.utc
        ) - datetime.timedelta(days=self.age_threshold)

        return (
            Character.objects.owned_by_user(user=user)
            .filter(
                details__birthday__lt=threshold_date,
            )
            .count()
            > 0
        )


class AssetFilter(BaseFilter):
    """
    AssetFilter
    """

    assets = models.ManyToManyField(
        EveType,
        help_text=_("User must possess <strong>one</strong> of the selected assets."),
    )

    @property
    def name(self):
        """
        Filter name
        :return:
        """

        return "Member Audit Asset"

    def process_filter(self, user: User):
        """
        Processing filter
        :param user:
        :return:
        """

        characters = Character.objects.owned_by_user(user=user)

        return (
            CharacterAsset.objects.filter(
                character__in=characters, eve_type__in=self.assets.all()
            ).count()
            > 0
        )


class ComplianceFilter(BaseFilter, SingletonModel):
    """
    ComplianceFilter
    """

    def process_filter(self, user: User):
        """
        Processing filter
        :param user:
        :return:
        """

        is_compliant = General.compliant_users().filter(pk=user.pk).exists()

        return is_compliant


class SkillPointFilter(BaseFilter):
    """
    SkillPointFilter
    """

    skill_point_threshold = models.PositiveBigIntegerField(
        help_text=_("Minimum allowable skillpoints."),
    )

    @property
    def name(self):
        """
        Filter name
        :return:
        """

        return (
            "Member Audit Skill Points "
            f"[sp={humanize.intword(self.skill_point_threshold)}]"
        )

    def process_filter(self, user: User):
        """
        Processing filter
        :param user:
        :return:
        """

        return (
            Character.objects.owned_by_user(user=user)
            .filter(
                skillpoints__total__gt=self.skill_point_threshold,
            )
            .count()
            > 0
        )


class SkillSetFilter(BaseFilter):
    """
    SkillSetFilter
    """

    skill_sets = models.ManyToManyField(
        SkillSet,
        help_text=_(
            "Users must possess all of the skills in <strong>one</strong> of the "
            "selected skillsets."
        ),
    )

    @property
    def name(self):
        """
        Filter name
        :return:
        """

        return "Member Audit Skill Set"

    def process_filter(self, user: User):
        """
        Processing filter
        :param user:
        :return:
        """

        characters = Character.objects.owned_by_user(user=user)

        for character in characters:
            for check in character.skill_set_checks.filter(
                skill_set__in=self.skill_sets.all()
            ):
                if check.failed_required_skills.count() == 0:
                    return True

        return False



# Generated by Django 3.1.2 on 2021-01-05 18:07

# Django
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("eveuniverse", "0004_effect_longer_name"),
        ("memberaudit", "0003_stats_n_skillset_autocomplete"),
    ]

    operations = [
        migrations.CreateModel(
            name="ActivityFilter",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "description",
                    models.CharField(
                        help_text="The filter description that is shown to end users.",
                        max_length=500,
                    ),
                ),
                (
                    "inactivity_threshold",
                    models.PositiveIntegerField(
                        help_text="Maximum allowable inactivity, in <strong>days</strong>."
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="AgeFilter",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "description",
                    models.CharField(
                        help_text="The filter description that is shown to end users.",
                        max_length=500,
                    ),
                ),
                (
                    "age_threshold",
                    models.PositiveIntegerField(
                        help_text="Minimum allowable age, in <strong>days</strong>."
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="ComplianceFilter",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "description",
                    models.CharField(
                        help_text="The filter description that is shown to end users.",
                        max_length=500,
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="SkillPointFilter",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "description",
                    models.CharField(
                        help_text="The filter description that is shown to end users.",
                        max_length=500,
                    ),
                ),
                (
                    "skill_point_threshold",
                    models.PositiveBigIntegerField(
                        help_text="Minimum allowable skillpoints."
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="SkillSetFilter",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "description",
                    models.CharField(
                        help_text="The filter description that is shown to end users.",
                        max_length=500,
                    ),
                ),
                (
                    "skill_sets",
                    models.ManyToManyField(
                        help_text="Users must possess all of the skills in <strong>one</strong> of the selected skillsets.",
                        to="memberaudit.SkillSet",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="AssetFilter",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "description",
                    models.CharField(
                        help_text="The filter description that is shown to end users.",
                        max_length=500,
                    ),
                ),
                (
                    "assets",
                    models.ManyToManyField(
                        help_text="User must possess <strong>one</strong> of the selected assets.",
                        to="eveuniverse.EveType",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
    ]



"""
App's base definitions
"""

# Django
from django.apps import AppConfig


class MemberAuditSecureGroupsConfig(AppConfig):
    """
    MemberAuditSecureGroupsConfig
    """

    name = "memberaudit_securegroups"
    label = "memberaudit_securegroups"
    verbose_name = "Member Audit Secure Groups Integration"



"""
Admin pages
"""

# Third Party
import humanize

# Django
from django.contrib import admin

# Memberaudit Securegroups
from memberaudit_securegroups.models import (
    ActivityFilter,
    AgeFilter,
    AssetFilter,
    ComplianceFilter,
    SkillPointFilter,
    SkillSetFilter,
)


class SingletonModelAdmin(admin.ModelAdmin):
    """
    Prevents Django admin users deleting the singleton or adding extra rows.
    """

    actions = None  # Removes the default delete action.

    def has_delete_permission(self, request, obj=None):
        return False

    def has_add_permission(self, request):
        return self.model.objects.all().count() == 0


@admin.register(ActivityFilter)
class ActivityFilterAdmin(admin.ModelAdmin):
    """
    ActivityFilterAdmin
    """

    list_display = ("description", "_inactivity_threshold")

    def _inactivity_threshold(self, obj):
        return f"{obj.inactivity_threshold:d} days"


@admin.register(AgeFilter)
class AgeFilterAdmin(admin.ModelAdmin):
    """
    AgeFilterAdmin
    """

    list_display = ("description", "_age_threshold")

    def _age_threshold(self, obj):
        return f"{obj.age_threshold:d} days"


@admin.register(AssetFilter)
class AssetFilterAdmin(admin.ModelAdmin):
    """
    AssetFilterAdmin
    """

    list_display = ("description",)
    filter_horizontal = ("assets",)


@admin.register(ComplianceFilter)
class ComplianceFilterAdmin(SingletonModelAdmin):
    """
    ComplianceFilterAdmin
    """

    list_display = ("description",)


@admin.register(SkillPointFilter)
class SkillPointFilterAdmin(admin.ModelAdmin):
    """
    SkillPointFilterAdmin
    """

    list_display = ("description", "_skill_point_threshold")

    def _skill_point_threshold(self, obj):
        return f"{humanize.intword(obj.skill_point_threshold)} skill points"


@admin.register(SkillSetFilter)
class SkillSetFilterAdmin(admin.ModelAdmin):
    """
    SkillSetFilterAdmin
    """

    list_display = ("description",)
    filter_horizontal = ("skill_sets",)
